let byte_size = 8 

let push_byte_as_bits = Pack.push_int_as_bits byte_size

let pop_bits_as_byte = Pack.pop_bits_as_int byte_size

(* [to_bits k is] is a consumer of the MyStream.t [is] and a producer of a new
   MyStream.t. It converts the [k]-bit integer MyStream.t [is] to a MyStream.t of bits. *)
let rec to_bits (is : int MyStream.t) : bool MyStream.t =
  lazy (
    match Lazy.force is with
    | MyStream.Nil          -> 
        MyStream.Nil
    | MyStream.Cons (x, xs) -> 
        Lazy.force (push_byte_as_bits x (to_bits xs))
  )

(* Alternative definition using [map_one_many] *)
let to_bits is =
  MyStream.map_one_many push_byte_as_bits is

(* [from_bits k bs] is a consumer of the MyStream.t [bs] and a producer of a new
   MyStream.t. It converts the bit MyStream.t [bs] into a MyStream.t of [k]-bit integers.
   If the number of elements in the MyStream.t [bs] is not a multiple of [k], an
   appropriate number of zero bits are inserted at the end as padding. *)
let rec from_bits (bs : bool MyStream.t) : int MyStream.t =
  lazy (
    match Lazy.force bs with
    | MyStream.Nil         -> 
        MyStream.Nil
    | MyStream.Cons (_, _) -> 
        let y, bs = pop_bits_as_byte bs in
        MyStream.Cons (y, from_bits bs)
  )
  
(* Alternative definition using [map_many_one] *)
let from_bits bs =
  MyStream.map_many_one pop_bits_as_byte bs
